from dialog_api import messaging_pb2


class DocumentExtPhoto:
    def __init__(self, height: int, width: int) -> None:
        self.height = height
        self.width = width

    @classmethod
    def from_api(cls, photo: messaging_pb2.DocumentExPhoto) -> 'DocumentExtPhoto':
        return cls(photo.h, photo.w)

    def __dict__(self):
        return {"height": self.height, "width": self.width}

    def __str__(self):
        return "DocumentExtPhoto({})".format(self.__dict__())


class DocumentExtVideo:
    def __init__(self, duration: int, height: int, width: int) -> None:
        self.duration = duration
        self.height = height
        self.width = width

    @classmethod
    def from_api(cls, video: messaging_pb2.DocumentExVideo) -> 'DocumentExtVideo':
        return cls(video.duration, video.h, video.w)

    def __dict__(self):
        return {"duration": self.duration, "height": self.height, "width": self.width}

    def __str__(self):
        return "DocumentExtVideo({})".format(self.__dict__())


class DocumentExtVoice:
    def __init__(self, duration: int) -> None:
        self.duration = duration

    @classmethod
    def from_api(cls, voice: messaging_pb2.DocumentExVoice) -> 'DocumentExtVoice':
        return cls(voice.duration)

    def __dict__(self):
        return {"duration": self.duration}

    def __str__(self):
        return "DocumentExtVoice({})".format(self.__dict__())


class DocumentExt:
    def __init__(self, photo: DocumentExtPhoto, video: DocumentExtVideo, voice: DocumentExtVoice) -> None:
        self.photo = photo
        self.video = video
        self.voice = voice

    @classmethod
    def from_api(cls, ext: messaging_pb2.DocumentEx) -> 'DocumentExt':
        return cls(DocumentExtPhoto.from_api(ext.photo), DocumentExtVideo.from_api(ext.video),
                   DocumentExtVoice.from_api(ext.voice))

    def __dict__(self):
        return {"photo": self.photo.__dict__(), "video": self.video.__dict__(), "voice": self.voice.__dict__(), }

    def __str__(self):
        return "DocumentExt({})".format(self.__dict__())


class DocumentMessage:
    def __init__(self, file_id: int, access_hash: int, file_size: int, mime_type: str, name: str, ext: DocumentExt) -> None:
        self.file_id = file_id
        self.access_hash = access_hash
        self.file_size = file_size
        self.mime_type = mime_type
        self.name = name
        self.ext = ext

    @classmethod
    def from_api(cls, document: messaging_pb2.DocumentMessage) -> 'DocumentMessage':
        return cls(document.file_id, document.access_hash, document.file_size, document.mime_type, document.name,
                   DocumentExt.from_api(document.ext))

    def __dict__(self):
        return {"file_id": self.file_id, "access_hash": self.access_hash, "file_size": self.file_size,
                "mime_type": self.mime_type, "name": self.name, "ext": self.ext.__dict__()}

    def __str__(self):
        return "DocumentMessage({})".format(self.__dict__())



from dialog_api.messaging_pb2 import LISTLOADMODE_UNKNOWN, LISTLOADMODE_FORWARD, LISTLOADMODE_BACKWARD, LISTLOADMODE_BOTH


class ListLoadMode:
    LISTLOADMODE_UNKNOWN = LISTLOADMODE_UNKNOWN
    LISTLOADMODE_FORWARD = LISTLOADMODE_FORWARD
    LISTLOADMODE_BACKWARD = LISTLOADMODE_BACKWARD
    LISTLOADMODE_BOTH = LISTLOADMODE_BOTH



from typing import List

from dialog_api import messaging_pb2

from dialog_bot_sdk.entities.Peer import Peer, PeerType
from dialog_bot_sdk.entities.UUID import UUID
from dialog_bot_sdk.entities.message.Message import MessageContent


class UpdateMessage:
    def __init__(self, peer: Peer, sender_peer: Peer, mid: UUID, message: MessageContent, forward: List[UUID],
                 reply: List[UUID], previous_mid: UUID, date: int) -> None:
        self.peer = peer
        self.sender_peer = sender_peer
        self.mid = mid
        self.message = message
        self.forward = forward
        self.reply = reply
        self.previous_mid = previous_mid
        self.date = date

    @classmethod
    def from_api(cls, update: messaging_pb2.UpdateMessage) -> 'UpdateMessage':
        return cls(Peer.from_api(update.peer), Peer(update.sender_uid, PeerType.PEERTYPE_PRIVATE),
                   UUID.from_api(update.mid), MessageContent.from_api(update.message),
                   [UUID.from_api(x) for x in update.forward.mids], [UUID.from_api(x) for x in update.reply.mids],
                   UUID.from_api(update.previous_mid), update.date)

    def __dict__(self):
        return {"peer": self.peer.__dict__(), "sender_peer": self.sender_peer.__dict__(),
                "mid": self.mid.__str__(), "message": self.message.__dict__(),
                "forward": [x.__str__() for x in self.forward], "reply": [x.__str__() for x in self.reply],
                "previous_mid": self.previous_mid.__str__(), "date": self.date}

    def __str__(self):
        return "UpdateMessage({})".format(self.__dict__())



from dialog_api import media_and_files_pb2


class FileLocation:
    def __init__(self, file_id: int, access_hash: int) -> None:
        self.file_id = file_id
        self.access_hash = access_hash

    def to_api(self) -> media_and_files_pb2.FileLocation:
        return media_and_files_pb2.FileLocation(file_id=self.file_id, access_hash=self.access_hash)

    @classmethod
    def from_api(cls, location: media_and_files_pb2.FileLocation) -> 'FileLocation':
        return cls(location.file_id, location.access_hash)

    def __dict__(self):
        return {"file_id": self.file_id, "access_hash": self.access_hash}

    def __str__(self):
        return "FileLocation({})".format(self.__dict__())



class UserStatus:
    USERSTATUSTYPE_UNKNOWN = 0
    USERSTATUSTYPE_UNSET = 1
    USERSTATUSTYPE_AWAY = 2
    USERSTATUSTYPE_DONOTDISTURB = 3
    USERSTATUSTYPE_INVISIBLE = 4
    USERSTATUSTYPE_BUSY = 5


from typing import List

from dialog_api import profile_pb2, users_pb2
from google.protobuf.wrappers_pb2 import StringValue

from dialog_bot_sdk.entities.Sex import Sex
from dialog_bot_sdk.entities.UserStatus import UserStatus

from dialog_bot_sdk.entities.Avatar import Avatar

from dialog_bot_sdk.utils import async_dec

from dialog_bot_sdk.service import ManagedService


class Profile(ManagedService):
    """Class for changing bot profile.

    """
    @async_dec()
    def edit_name(self, name: str) -> None:
        """Edit bot name.

        :param name: new bot name
        :return: None
        """
        self.internal.profile.EditName(profile_pb2.RequestEditName(name=name))

    @async_dec()
    def edit_nickname(self, nick: str) -> None:
        """Edit bot nickname (shortname).

        :param nick: new bot nickname.
        :return: None
        """
        self.internal.profile.EditNickName(profile_pb2.RequestEditNickName(nickname=StringValue(value=nick)))

    @async_dec()
    def check_nickname(self, nick: str) -> bool:
        """Validate nickname (shortname).

        :param nick: nickname.
        :return: True or False
        """
        try:
            return self.internal.profile.CheckNickName(profile_pb2.RequestCheckNickName(nickname=nick))
        except:
            return False

    @async_dec()
    def edit_about(self, about: str) -> None:
        """Edit bot about.

        :param about: new bot about.
        :return:
        """
        self.internal.profile.EditAbout(profile_pb2.RequestEditAbout(about=StringValue(value=about)))

    @async_dec()
    def edit_avatar(self, file: str) -> Avatar or None:
        """Edit bot avatar.

        :param file: new bot avatar (path to file).
        :return: Avatar or None.
        """
        location = self.internal.uploading.upload_file(file).wait()
        if location is None:
            return None
        return self.internal.profile.EditAvatar(profile_pb2.RequestEditAvatar(file_location=location.to_api()))

    @async_dec()
    def remove_avatar(self) -> None:
        """Remove bot avatar.

        :return: None.
        """
        self.internal.profile.RemoveAvatar(profile_pb2.RequestRemoveAvatar())

    @async_dec()
    def edit_time_zone(self, tz: str) -> None:
        """Edit bot timezone.

        :param tz: new bot timezone.
        :return: None.
        """
        self.internal.profile.EditMyTimeZone(profile_pb2.RequestEditMyTimeZone(tz=tz))

    @async_dec()
    def edit_preferred_languages(self, preferred_languages: List[str]) -> None:
        """Edit bot preferred languages.

        :param preferred_languages: new bot preferred languages.
        :return: None.
        """
        self.internal.profile.EditMyPreferredLanguages(
            profile_pb2.RequestEditMyPreferredLanguages(preferred_languages=preferred_languages)
        )

    @async_dec()
    def edit_sex(self, sex: Sex) -> None:
        """Edit bot sex.

        :param sex: new bot sex.
        :return: None.
        """
        self.internal.profile.EditSex(profile_pb2.RequestEditSex(sex=sex))

    @async_dec()
    def edit_custom_profile(self, custom_profile: str) -> None:
        """Edit bot sex.

        :param custom_profile: new bot custom profile.
        :return: None.
        """
        self.internal.profile.EditCustomProfile(profile_pb2.RequestEditCustomProfile(custom_profile=custom_profile))

    @async_dec()
    def edit_user_status(self, type: UserStatus, text: str) -> None:
        """Edit bot sex.

        :param type: new bot status type.
        :param text: new bot status text.
        :return: None.
        """
        self.internal.profile.ChangeUserStatus(
            profile_pb2.RequestChangeUserStatus(status=users_pb2.UserStatus(type=type, text=StringValue(value=text)))
        )



import unittest

from tests.test_updates import TestUpdates
from tests.test_users import TestUsers
from tests.test_groups import TestGroups
from tests.test_bot import TestBot
from tests.test_messaning import TestMessaging

if __name__ == '__main__':
    unittest.main()
